/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * ChangeTracker.cpp
 *
 *  Created on: Mar 1, 2010
 *      Author: metzler
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

//#include "CodePatterns/MemDebug.hpp"

#include "Atom/AtomObserver.hpp"
#include "CodePatterns/Singleton_impl.hpp"
#include "CodePatterns/Log.hpp"
#include "CodePatterns/Verbose.hpp"
#include "Parser/ChangeTracker.hpp"
#include "World.hpp"

/**
 * Constructor. Signs on as an observer for the World.
 */
ChangeTracker::ChangeTracker() :
  Observable("ChangeTracker"),
  isConsistent(true)
{
  World::getInstance().signOn(this);
  AtomObserver::getInstance().signOn(this);
}

/**
 * Destructor. Signs off from the World.
 */
ChangeTracker::~ChangeTracker()
{
  AtomObserver::getInstance().signOff(this);
  World::getInstance().signOff(this);
}

/**
 * With this, the World can update the change tracker's state.
 */
void ChangeTracker::update(Observable *publisher) {
  isConsistent = false;
}

/**
 * Gets whether there are non-saved changes.
 *
 * \param true if there are any changes, false otherwise
 */
bool ChangeTracker::hasChanged() {
  return !isConsistent;
}

/**
 * Tells all observers (which are the different parsers) that they are supposed
 * to save the current state.
 */
void ChangeTracker::saveStatus() {
  if (hasChanged()) {
    LOG(0, "Saving changes.");
    notifyAll();
  }
}

CONSTRUCT_SINGLETON(ChangeTracker)
